///\file
///\brief implementation file for psycle::host::RiffFile.
#include <WTypes.h>
#include "FileIO.hpp"
		unsigned long RiffFile::FourCC(char const *psName)
		{
			long retbuf = 0x20202020; // four spaces (padding)
			char *ps = ((char *)&retbuf);
			// Remember, this is Intel format!
			// The first character goes in the LSB
			for(int i(0); i < 4 && psName[i]; ++i) *ps++ = psName[i];
			return retbuf;
		}

		bool RiffFile::Open(std::string psFileName)
		{
			unsigned long bytesRead;
			szName = psFileName;
			_modified = false;
			_handle = ::CreateFile(psFileName.c_str(), GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
			if(_handle == INVALID_HANDLE_VALUE) return false;

/*			Don't need the header now, so this is commented out.			
			if(!ReadFile(_handle, &_header, sizeof(_header), &bytesRead, 0))
			{
				CloseHandle(_handle);
				return false;
			}
*/
			return true;
		}

		bool RiffFile::Create(std::string psFileName, bool overwrite)
		{
			unsigned long bytesWritten;
			szName = psFileName;
			_modified = false;
			_handle = ::CreateFile(psFileName.c_str(), GENERIC_READ | GENERIC_WRITE, 0, 0, overwrite ? CREATE_ALWAYS : CREATE_NEW, FILE_ATTRIBUTE_NORMAL, 0);
			if(_handle == INVALID_HANDLE_VALUE) return false;
			_header._id = FourCC("RIFF");
			_header._size = 0;
			if(!WriteFile(_handle, &_header, sizeof(_header), &bytesWritten, 0))
			{
				CloseHandle(_handle);
				return false;
			}
			return true;
		}

		bool RiffFile::Close()
		{
			unsigned long bytesWritten;
			if (_modified)
			{
				Seek(0);
				WriteFile(_handle, &_header, sizeof(_header), &bytesWritten, 0);
				_modified = false;
			}
			bool b = CloseHandle(_handle);
			_handle = INVALID_HANDLE_VALUE;
			return b;
		}

		bool RiffFile::Read(void* pData, unsigned long numBytes)
		{
			unsigned long bytesRead;
			if(!ReadFile(_handle, pData, numBytes, &bytesRead, 0)) return false;
			return (bytesRead == numBytes);
		}

		bool RiffFile::Write(const void * const pData, unsigned long numBytes)
		{
			unsigned long bytesWritten;
			if(!WriteFile(_handle, pData, numBytes, &bytesWritten, 0)) return false;
			_modified = true;
			_header._size += bytesWritten;
			return (bytesWritten == numBytes);
		}

		bool RiffFile::Expect(void* pData, unsigned long numBytes)
		{
			unsigned long bytesRead;
			unsigned char c;
			while(numBytes-- != 0)
			{
				if(!ReadFile(_handle, &c, sizeof(c), &bytesRead, 0)) return false;
				if(c != *((char*)pData)) return false;
				pData = (char*)pData + 1;
			}
			return true;
		}

		long RiffFile::Seek(long offset)
		{
			return SetFilePointer(_handle, offset, 0, FILE_BEGIN);
		}

		long RiffFile::Skip(long numBytes)
		{
			return SetFilePointer(_handle, numBytes, 0, FILE_CURRENT); 
		}

		bool RiffFile::Eof()
		{
			return false;
		}

		long RiffFile::GetPos()
		{
			return SetFilePointer(_handle, 0, 0, FILE_CURRENT);
		}

		long RiffFile::FileSize()
		{
			const long init = GetPos();
			const long end = SetFilePointer(_handle,0,0,FILE_END);
			SetFilePointer(_handle,init,0,FILE_BEGIN);
			return end;
		}

		bool RiffFile::ReadString(std::string &result)
		{
			result="";
			char c;
			while(true)
			{
				if (Read(&c, sizeof(c)))
				{
					if(c == 0 || c == '\n') {
						return true;
					}
					result+=c;
				}
				else return false;
			}
		}

		bool RiffFile::ReadString(char* pData, unsigned long maxBytes)
		{
			if(maxBytes > 0)
			{
				memset(pData,0,maxBytes);

				char c;
				for(unsigned long index = 0; index < maxBytes; index++)
				{
					if (Read(&c, sizeof(c)))
					{
						pData[index] = c;
						if(c == 0) return true;
					}
					else return false;
				}
				do
				{
					if(!Read(&c, sizeof(c))) return false; //\todo : return false, or return true? the string is read already. it could be EOF.
				} while(c != 0);
				return true;
			}
			return false;
		}

