#include <WTypes.h>
#include "hoster.h"
#include "msdnHelpers.h"

typedef winampDSPHeader* (*winampDSPGetHeaderTypeWithWin)(HWND);
typedef winampDSPHeader* (*winampDSPGetHeaderTypeWithoutWin)();

Hoster::Hoster(std::string plugname):
module(nullptr),
dllmodule(nullptr)
{
    if (!(dllmodule = LoadLibrary(plugname.c_str())))
        return;

    mainWnd = GetMainWindowOfProc(GetCurrentProcessId());

/*    try {
        winampDSPGetHeaderTypeWithWin GetHeader;
        if (!(GetHeader = reinterpret_cast<winampDSPGetHeaderTypeWithWin>(GetProcAddress(dllmodule, "winampDSPGetHeader2")))) {
            FreeLibrary(dllmodule);
            dllmodule = nullptr;
            return;
        }
        if (!(header = GetHeader(mainWnd))) {
            FreeLibrary(dllmodule);
            dllmodule = nullptr;
            return;
        }
    }
    catch (...) */{
        try {
            winampDSPGetHeaderTypeWithoutWin GetHeader;
            if (!(GetHeader = reinterpret_cast<winampDSPGetHeaderTypeWithoutWin>(GetProcAddress(dllmodule, "winampDSPGetHeader2")))) {
                FreeLibrary(dllmodule);
                dllmodule = nullptr;
                return;
            }
            if (!(header = GetHeader())) {
                FreeLibrary(dllmodule);
                dllmodule = nullptr;
                return;
            }
        }
        catch (...) {
            FreeLibrary(dllmodule);
            dllmodule = nullptr;
            return;
        }
    }
}

Hoster::~Hoster(void)
{
    Quit();
}

void Hoster::Init(int idx) {
    if (module) {
        module->Quit(module);
        module = nullptr;
    }
    try {
        if (!(module = header->getModule(idx))) {
            FreeLibrary(dllmodule);
            dllmodule = nullptr;
            return;
        }
    }
    catch (...) {
        FreeLibrary(dllmodule);
        dllmodule = nullptr;
        return;
    }

    module->hDllInstance = dllmodule;
    module->hwndParent = mainWnd;
    module->Init(module);
}
void Hoster::Quit() {
    if (module)
    {
        ///\ dsp_pacemaker.dll hangs when calling Quit.
        // It could be because hwndParent is null, but i don't know it.
        module->Quit(module);
    }
    if (dllmodule) {
        FreeLibrary(dllmodule);
    }
}
// TODO: In order to allow different bitdepths, the type of samples needs to change.
void Hoster::Process(short int *samples, int numSamples, int bits, bool stereo, int srate)
{
	assert(module);

	///\todo: dsp effects can generate a different number of Samples than numSamples.
	// This is not supported with VST, except when using Offline processing.
	// for now, we just discard the extra numsamples, and just set to null the rest.
	int realsamples = module->ModifySamples(module,samples,numSamples,bits,stereo?2:1,srate);
	if ( realsamples < numSamples ) memset(samples+realsamples,0,(numSamples-realsamples)*(bits/(stereo?4:8)));
}