#include <WTypes.h>
#include "awinamp.h"
#include "FileIO.hpp"


//-------------------------------------------------------------------------------------------------------
AudioEffect* createEffectInstance (audioMasterCallback audioMaster)
{
	return new AWinamp (audioMaster);
}

//-------------------------------------------------------------------------------------------------------
AWinamp::AWinamp (audioMasterCallback audioMaster)
: AudioEffectX (audioMaster, 1, 1)	// 1 program, 1 parameter only
, buffer(nullptr)
, fGain(1.0f)			// default to 0 dB
, iBits(16)
, bStereo(true)
, pHoster(nullptr)
{
	setNumInputs (2);		// stereo in
	setNumOutputs (2);		// stereo out


	// WARNING!!!!! This should be changed if this plugin is going to be used publicly!!!
	setUniqueID ('WadH');	// identify

	setBlockSize(blockSize);
    setBlockSize(blockSize);

	canProcessReplacing ();	// supports replacing output
	vst_strncpy (programName, "Default", kVstMaxProgNameLen);	// default program name
}

//-------------------------------------------------------------------------------------------------------
AWinamp::~AWinamp ()
{
	if ( buffer ) delete buffer;
}
void AWinamp::open ()
{
	std::string confname;
	if ( getDirectory() != NULL)
	{
		confname = reinterpret_cast<const char*>(getDirectory());
	}
	else
	{
		char c[1 << 10];
		c[0]='\0';
		::GetModuleFileName(0, c, sizeof c);
		confname = c;
        confname = confname.substr(0, confname.rfind('\\'));
	}
	confname = confname + "\\plugin.ini";

	RiffFile configini;
	if ( ! configini.Open(confname) ) 
		return;

	std::string plugname;
	configini.ReadString(plugname);
	configini.Close();

	pHoster = new Hoster(plugname);
    //TODO: ShellPlugin
    pHoster->Init(0);

    Guiwrapper* gui = new Guiwrapper(this);
    gui->setHost(pHoster);
    setEditor(gui);
}
void AWinamp::close ()
{
	if (pHoster)
		delete pHoster;
}
void AWinamp::setBlockSize (VstInt32 blockSize)
{
	AudioEffectX::setBlockSize(blockSize);
	if (buffer) delete buffer;

	buffer = new short int[blockSize*4]; // stereo, and twice the samples to avoid crashes on resampling DSPs ( so 2*2 = 4 )
}

//-------------------------------------------------------------------------------------------------------
void AWinamp::setProgramName (char* name)
{
	vst_strncpy (programName, name, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void AWinamp::getProgramName (char* name)
{
	vst_strncpy (name, programName, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void AWinamp::setParameter (VstInt32 index, float value)
{
	switch(index)
	{
	case 0:fGain = value;
		break;
	default:
		break;
	}
}

//-----------------------------------------------------------------------------------------
float AWinamp::getParameter (VstInt32 index)
{
	switch(index)
	{
	case 0:return fGain;
		break;
	default:
        return 0.f;
		break;
	}
}

//-----------------------------------------------------------------------------------------
void AWinamp::getParameterName (VstInt32 index, char* label)
{
	switch(index)
	{
	case 0: vst_strncpy (label, "Gain", kVstMaxParamStrLen);
		break;
	default:
		break;
	}
}

//-----------------------------------------------------------------------------------------
void AWinamp::getParameterDisplay (VstInt32 index, char* text)
{
	switch(index)
	{
	case 0:dB2string (fGain, text, kVstMaxParamStrLen);
		break;
	default:
		break;
	}	
}

//-----------------------------------------------------------------------------------------
void AWinamp::getParameterLabel (VstInt32 index, char* label)
{
	switch(index)
	{
	case 0:vst_strncpy (label, "dB", kVstMaxParamStrLen);
		break;
	default:
		break;
	}
}

//------------------------------------------------------------------------
bool AWinamp::getEffectName (char* name)
{
	std::string mystring = "WA DSP";
    if (pHoster && pHoster->module)
	{
		pHoster->GetInfo(mystring);
    } else {
        mystring = "error";
    }
	vst_strncpy (name, mystring.c_str(), kVstMaxProductStrLen);
	return true;
}

//------------------------------------------------------------------------
bool AWinamp::getProductString (char* text)
{
	std::string mystring = "WA DSP";
    if (pHoster && pHoster->module)
    {
		pHoster->GetInfo(mystring);
	} else {
        mystring = "error";
    }

	vst_strncpy (text, mystring.c_str(), kVstMaxProductStrLen);
	return true;
}

//------------------------------------------------------------------------
bool AWinamp::getVendorString (char* text)
{
	vst_strncpy (text, "Public Domain", kVstMaxVendorStrLen);
	return true;
}

//-----------------------------------------------------------------------------------------
VstInt32 AWinamp::getVendorVersion ()
{ 
	return 1000; 
}
// TODO: Possibly implement multi-dsp plugins using the shell category and the getCurrentUniqueID
// Check audioeffecx.cpp , bottom for an example. Not all host might suppor it. */
/*VstPlugCategory AWinamp::getPlugCategory()
{
    return kPlugCategShell;
}*/

//-----------------------------------------------------------------------------------------
void AWinamp::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames)
{
	float* in1  =  inputs[0];
	float* in2  =  inputs[1];
	float* out1 = outputs[0];
	float* out2 = outputs[1];

	QuantizeAndInterlace(in1,in2,buffer, sampleFrames);
    if (pHoster && pHoster->module) {
        pHoster->Process(buffer, sampleFrames, iBits, bStereo, static_cast<int>(getSampleRate()));
    }
	DeQuantizeAndDeInterlace(buffer,out1,out2, sampleFrames);

	while (--sampleFrames >= 0)
	{
		assert(*in1++==*out1);
		(*out1++) *= fGain;
		(*out2++) *= fGain;
	}
}

void AWinamp::QuantizeAndInterlace(float *pinleft, float *pinright, short int *piout, int c)
{
#define SHORT_MIN	-32768
#define SHORT_MAX	32767
	
	do
	{
		int l = static_cast<int>(*pinleft* 32768.f);
		if (l < SHORT_MIN)
		{
			l = SHORT_MIN;
		}
		else if (l > SHORT_MAX)
		{
			l = SHORT_MAX;
		}
		*piout++ = l;
		
		int r = static_cast<int>(*pinright* 32768.f);
		if (r < SHORT_MIN)
		{
			r = SHORT_MIN;
		}
		else if (r > SHORT_MAX)
		{
			r = SHORT_MAX;
		}
		*piout++ = r;

		pinleft++; pinright++;
	}
	while(--c);
}

void AWinamp::DeQuantizeAndDeInterlace(short int *pin, float *poutleft,float *poutright, int c)
{
	const float multiplier = 0.000030517578125f;
	do
	{
		*poutleft++ = *(pin++)*multiplier;
		*poutright++ = *(pin++)*multiplier;
	}
	while(--c);
}
