#pragma once
#include <windows.h>
#include <tlhelp32.h>

BOOL CALLBACK EnumThreadWndProc(_In_ HWND hWnd, _In_ LPARAM lParam)
{
    *((HWND *)lParam) = hWnd;

    HWND owner = GetWindow(hWnd, GW_OWNER);
    return (owner != nullptr);
}


HWND GetMainWindowOfProc(DWORD dwOwnerPID)
{
    HANDLE hThreadSnap = INVALID_HANDLE_VALUE;
    THREADENTRY32 te32;

    // Take a snapshot of all running threads  
    hThreadSnap = CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD, 0);
    if (hThreadSnap == INVALID_HANDLE_VALUE)
        return nullptr;

    // Fill in the size of the structure before using it. 
    te32.dwSize = sizeof(THREADENTRY32);

    // Retrieve information about the first thread,
    // and exit if unsuccessful
    if (!Thread32First(hThreadSnap, &te32))
    {
        CloseHandle(hThreadSnap);     // Must clean up the snapshot object!
        return nullptr;
    }

    // Now walk the thread list of the system,
    // and display information about each thread
    // associated with the specified process
    HWND hWnd = nullptr;
    do
    {
        if (te32.th32OwnerProcessID == dwOwnerPID)
        {
            EnumThreadWindows(te32.th32ThreadID, EnumThreadWndProc, (LPARAM)&hWnd);
            if (hWnd != nullptr) {
                break;
            }
        }
    } while (Thread32Next(hThreadSnap, &te32));

    //  Don't forget to clean up the snapshot object.
    CloseHandle(hThreadSnap);
    return hWnd;
}

